///////////////////////////////////////////////////////////////////////////////
// File: SocketServerCallback.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"
#include "JetByteTools/Admin/WinsockWrapper.h"

#include "SocketServerCallback.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/Exception.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::DumpData;
using JetByteTools::Core::CException;
using JetByteTools::Core::GetStringLength;

using JetByteTools::IO::CSmartBuffer;

using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::IAddress;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// CSocketServerCallback
///////////////////////////////////////////////////////////////////////////////

CSocketServerCallback::CSocketServerCallback(
   const string &welcomeMessage)
   :  m_welcomeMessage(welcomeMessage)
{

}

void CSocketServerCallback::OnConnectionEstablished(
   IStreamSocket &socket,
   const IAddress & /*address*/)
{
   Output(_T("OnConnectionEstablished"));

   socket.Write(m_welcomeMessage.c_str(), GetStringLength<DWORD>(m_welcomeMessage));

   socket.Read();
}

void CSocketServerCallback::OnReadCompleted(
   IStreamSocket &socket,
   CSmartBuffer &buffer)
{
   EchoMessage(socket, buffer);

   socket.Read();
}

void CSocketServerCallback::EchoMessage(
   IStreamSocket &socket,
   CSmartBuffer &buffer) const
{
   //DEBUG_ONLY(Output(_T("\r\n") + DumpData(buffer->GetMemory(), buffer->GetUsed(), 60, true)));

   socket.Write(buffer);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
