///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "SocketServer.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/Exception.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Milliseconds;

using JetByteTools::Core::Output;
using JetByteTools::Core::DumpData;
using JetByteTools::Core::CException;
using JetByteTools::Core::ToStringA;
using JetByteTools::Core::GetStringLength;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::CSmartBuffer;
using JetByteTools::IO::IIOPool;

using JetByteTools::Socket::CFilteringStreamSocketServerEx;
using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::IAllocateStreamSockets;
using JetByteTools::Socket::IFullAddress;
using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::ILimitConnections;
using JetByteTools::Socket::ListenBacklog;
using JetByteTools::Socket::NoZeroByteRead;

typedef JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter::TimerCallback TimerCallback;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

CSocketServer::CSocketServer(
   const string &welcomeMessage,
   const IFullAddress &address,
   const ListenBacklog listenBacklog,
   IIOPool &pool,
   IAllocateStreamSockets &socketAllocator,
   IAllocateBuffers &bufferAllocator,
   ILimitConnections &connectionLimiter)
   :  CFilteringStreamSocketServerEx(address, listenBacklog, *this, pool, socketAllocator, bufferAllocator, NoZeroByteRead, connectionLimiter),
      m_timeoutFilter(*this, m_timerQueue),
      m_welcomeMessage(welcomeMessage)
{

}

CSocketServer::~CSocketServer()
{
   WaitForShutdownToComplete();
}

void CSocketServer::OnConnectionEstablished(
   IStreamSocket &socket,
   const IAddress & /*address*/)
{
   Output(_T("OnConnectionEstablished"));

   if (socket.TryWrite(m_welcomeMessage.c_str(), GetStringLength<DWORD>(m_welcomeMessage)))
   {
      m_timeoutFilter.SetReadTimeout(socket, 5000, *this, 1);

      socket.TryRead();
   }
}

void CSocketServer::OnReadCompleted(
   IStreamSocket &socket,
   CSmartBuffer &buffer)
{
   EchoMessage(socket, buffer);

   socket.Read();
}

void CSocketServer::OnTimer(
   IStreamSocket &socket,
   TimerCallback::UserData userData)
{
   if (userData < 5)
   {
      const string message = "Read timeout! " + ToStringA(userData) + "\r\n";

      socket.Write(message.c_str(), GetStringLength<DWORD>(message));

      const Milliseconds timeout = static_cast<Milliseconds>(userData);

      m_timeoutFilter.SetSingleReadTimeout(socket, 1000 * timeout, *this, timeout + 1);
   }
   else
   {
      const string message = "Too many timeouts!\r\n";

      socket.Write(message.c_str(), GetStringLength<DWORD>(message));

      socket.Shutdown();
   }
}

void CSocketServer::EchoMessage(
   IStreamSocket &socket,
   CSmartBuffer &buffer) const
{
   //DEBUG_ONLY(Output(_T("Data Echoed -\r\n") + DumpData(buffer->GetMemory(), buffer->GetUsed(), 60, true)));

   socket.Write(buffer);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
