#pragma once
#ifndef ECHO_SERVER_TEST_TEST_MESSAGES_INCLUDED__
#define ECHO_SERVER_TEST_TEST_MESSAGES_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: TestMessages.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/WinsockWrapper.h"

#include "JetByteTools/CoreTools/Types.h"
#include "JetByteTools/CoreTools/tstring.h"
#include "JetByteTools/CoreTools/AtomicBool.h"
#include "JetByteTools/CoreTools/LockableObject.h"

#include <vector>
#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Socket
   {
      class IStreamSocket;
   }
}

class CMessageCollection;
class ITestMessage;

///////////////////////////////////////////////////////////////////////////////
// CTestMessages
///////////////////////////////////////////////////////////////////////////////

class CTestMessages
{
   public :

      typedef unsigned long DataLength;

      enum TrackPending
      {
         TrackPendingBytes,
         TrackPendingMessages
      };

      CTestMessages(
         size_t connectionIndex,
         size_t repeatCount,
         JetByteTools::Milliseconds transferRateMillis,
         CMessageCollection &messageCollection,
         bool sendAfterRecv,
         bool holdConnections,
         DataLength skipSignOnMessageBytes,
         bool fragments,
         bool multipleMessages,
         TrackPending trackPending);

      CTestMessages(
         const CTestMessages &rhs) = delete;

      ~CTestMessages();

      CTestMessages &operator=(
         const CTestMessages &rhs) = delete;

      void AddMessage(
         ITestMessage *pMessage);

      bool ValidateReceivedData(
         const BYTE *pData,
         DataLength dataLength,
         bool &messageComplete);

      bool WriteSome(
         JetByteTools::Socket::IStreamSocket &socket);

      void ConnectionComplete(
         bool succeeded);

      void MessagesComplete(
         bool succeeded);

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      bool Succeeded(
         StringErrors &errors) const;

      bool SendAfterRecv() const;

      JetByteTools::Milliseconds GetTransferRateMillis() const;

      bool HoldConnections() const;

      DWORD GetAverageRTT() const;

      DWORD GetMaxRTT() const;

      DWORD GetMinRTT() const;

   private :

      int GetNumberOfMessagesToSend(
         size_t numMessages,
         int minMessages) const;

      void InternalConnectionComplete(
         bool succeeded);

      JetByteTools::Core::CLockableObject m_lock;

      typedef std::vector<ITestMessage *> Messages;

      CMessageCollection &m_messageCollection;

      const size_t m_connectionIndex;

      const bool m_sendAfterRecv;

      const bool m_holdConnections;

      const DataLength m_skipSignOnMessageBytes;

      const bool m_fragments;

      const bool m_multipleMessages;

      Messages m_messages;

      size_t m_sendIndex;

      size_t m_receiveIndex;

      bool m_succeeded;

      bool m_connected;

      const size_t m_repeatCount;

      const JetByteTools::Milliseconds m_transferRateMillis;

      size_t m_currentSendIteration;

      size_t m_currentReceiveIteration;

      DataLength m_signOnBytesSkipped;

      JetByteTools::Core::CAtomicBool m_complete;

      JetByteTools::Core::CAtomicBool m_connectionComplete;

      JetByteTools::Core::CAtomicBool m_allDataSent;

      size_t m_dataPending;

      const TrackPending m_trackPending;
};

#endif // ECHO_SERVER_TEST_TEST_MESSAGES_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: TestMessages.h
///////////////////////////////////////////////////////////////////////////////
