#pragma once
#ifndef PACKET_ECHO_SERVER_SOCKET_SERVER_INCLUDED__
#define PACKET_ECHO_SERVER_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include <string>

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback
{
   public :

      CSocketServer(
         const std::string &welcomeMessage,
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      ~CSocketServer();

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      // Our business logic

      void EchoMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      JetByteTools::IO::IBuffer::BufferSize GetMinimumMessageSize() const;

      JetByteTools::IO::IBuffer::BufferSize GetMessageSize(
         const JetByteTools::IO::IBuffer &buffer) const;

      void ProcessDataStream(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      const std::string m_welcomeMessage;

      /// No copies do not implement
      CSocketServer(const CSocketServer &rhs);
      /// No copies do not implement
      CSocketServer &operator=(const CSocketServer &rhs);
};

#endif // PACKET_ECHO_SERVER_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
