///////////////////////////////////////////////////////////////////////////////
// File: IOPool.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "IOPool.h"

#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::_tstring;
using JetByteTools::Core::ToString;

///////////////////////////////////////////////////////////////////////////////
// CIOPool
///////////////////////////////////////////////////////////////////////////////

CIOPool::CIOPool(
   const ThreadCount numThreads,
   const bool log)
   :  JetByteTools::IO::CIOPool(*this, numThreads),
      m_log(log)
{

}

CIOPool::~CIOPool()
{
   WaitForShutdownToComplete();
}

void CIOPool::OnIOPoolError(
   const _tstring &message)
{
   OutputEx(_T("OnError - ") + message);
}

void CIOPool::OnIOPoolThreadCreated()
{
   if (m_log)
   {
      Output(_T("OnIOPoolThreadCreated"));
   }
}

void CIOPool::OnIOPoolThreadStarted()
{
   if (m_log)
   {
      Output(_T("OnIOPoolThreadStarted"));
   }
}

void CIOPool::OnIOPoolDispatch()
{
   if (m_log)
   {
      Output(_T("OnIOPoolDispatch"));
   }
}

void CIOPool::OnIOPoolWorkItemsDequeued(
   const DWORD numWorkItems)
{
   if (m_log)
   {
      Output(_T("OnIOPoolWorkItemsDequeued: ") + ToString(numWorkItems));
   }
}

void CIOPool::OnIOPoolThreadBeginProcessing()
{
   if (m_log)
   {
      Output(_T("OnIOPoolThreadBeginProcessing"));
   }
}

void CIOPool::OnIOPoolThreadEndProcessing()
{
   if (m_log)
   {
      Output(_T("OnIOPoolThreadEndProcessing"));
   }
}

void CIOPool::OnIOPoolThreadStopped()
{
   if (m_log)
   {
      Output(_T("OnIOPoolThreadStopped"));
   }
}

void CIOPool::OnIOPoolThreadDestroyed()
{
   if (m_log)
   {
      Output(_T("OnIOPoolThreadDestroyed"));
   }
}

///////////////////////////////////////////////////////////////////////////////
// End of file: IOPool.cpp
///////////////////////////////////////////////////////////////////////////////
