#pragma once
#ifndef THREAD_POOL_LARGE_PACKET_ECHO_SERVER_SOCKET_SERVER_INCLUDED__
#define THREAD_POOL_LARGE_PACKET_ECHO_SERVER_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include "JetByteTools/SocketTools/ReadSequencingStreamSocketConnectionFilter.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Win32
   {
      class ICriticalSectionFactory;
      class IIndexedOpaqueUserData;
   }

   namespace Socket
   {
      class IAllocateSequencedStreamSockets;
   }
}

class CThreadPool;
class CMessageAccumulator;

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::Socket::CFilteringStreamSocketServerEx,
   private CStreamSocketServerExCallback
{
   public :

      CSocketServer(
         CThreadPool &businessLogicPool,
         const JetByteTools::IO::IBuffer::BufferSize maxMessageSize,
         const bool issueEarlyRead,
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &ioPool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         const JetByteTools::Socket::SocketBufferSize recvBufferSize,
         const JetByteTools::Socket::SocketBufferSize sendBufferSize,
         const JetByteTools::Socket::ZeroByteReadConfiguration zeroByteReadConfiguration,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      CSocketServer(
         const CSocketServer &rhs) = delete;

      ~CSocketServer();

      CSocketServer &operator=(
         const CSocketServer &rhs) = delete;

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      // Our business logic

      void ProcessDataStream(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      void ProcessCommand(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      CThreadPool &m_pool;

      void SetPerConnectionData(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         CMessageAccumulator *pData) const;

      CMessageAccumulator &GetPerConnectionData(
         const JetByteTools::Core::IIndexedOpaqueUserData &userData) const;

      const JetByteTools::IO::IBuffer::BufferSize m_maxMessageLength;

      const bool m_issueEarlyRead;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;

      JetByteTools::Socket::CReadSequencingStreamSocketConnectionFilter m_readSequenceFilter;
};

#endif // THREAD_POOL_LARGE_PACKET_ECHO_SERVER_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
