#pragma once
#ifndef THREAD_POOL_SIMPLE_PROTOCOL_SERVER_THREAD_POOL_INCLUDED__
#define THREAD_POOL_SIMPLE_PROTOCOL_SERVER_THREAD_POOL_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ThreadPool.h
/////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/ThreadPool.h"
#include "JetByteTools/CoreTools/IThreadPoolWorkerThreadFactory.h"
#include "JetByteTools/CoreTools/IThreadPoolWorkerThreadCallback.h"
#include "JetByteTools/CoreTools/IMonitorThreadPool.h"

#include "JetByteTools/IOTools/SmartBuffer.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Core
   {
      class IProvideUserData;
      class IIndexedOpaqueUserData;
   }

   namespace Socket
   {
      class IAddress;
      class IStreamSocket;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CThreadPool
///////////////////////////////////////////////////////////////////////////////

class CThreadPool :
   private JetByteTools::Core::IThreadPoolWorkerThreadFactory,
   private JetByteTools::Core::IThreadPoolWorkerThreadCallback,
   private JetByteTools::Core::IMonitorThreadPool
{
   public:

      typedef JetByteTools::Core::CThreadPool::ThreadCount ThreadCount;

      CThreadPool(
         JetByteTools::Core::IProvideUserData &userDataProvider,
         ThreadCount initialThreads,
         ThreadCount minThreads,
         ThreadCount maxThreads,
         ThreadCount maxDormantThreads,
         JetByteTools::Milliseconds poolMaintPeriod,
         JetByteTools::Milliseconds dispatchTimeout);

      CThreadPool(
         const CThreadPool &rhs) = delete;

      CThreadPool &operator=(
         const CThreadPool &rhs) = delete;

      void Start();

      void WaitForShutdownToComplete(
         JetByteTools::Milliseconds forceTerminationAfter = INFINITE);

      void DispatchConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address);

      void DispatchReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData);

   private :

      enum DispatchEvents
      {
         ConnectionEstablished,
         ReadCompleted
      };

      class CPerConnectionData
      {
         public :

            CPerConnectionData(
               const JetByteTools::Core::_tstring &address);

            CPerConnectionData(
               const CPerConnectionData &rhs) = delete;

            CPerConnectionData &operator=(
               const CPerConnectionData &rhs) = delete;

            void UseConnection();

            JetByteTools::Core::_tstring GetConnectionDetails() const;

         private :

            long m_numUses;

            const JetByteTools::Core::_tstring m_address;
      };

      CPerConnectionData &GetPerConnectionData(
         const JetByteTools::Core::IIndexedOpaqueUserData &userData) const;

      void ProcessMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::IO::CSmartBuffer &buffer) const;

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket);

      void DoDispatch(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::IBuffer *pBuffer,
         DispatchEvents event);

      // Implement IThreadPoolWorkerThreadFactory

      JetByteTools::Core::IThreadPoolWorkerThreadCallback &CreateWorkerThread() override;

      void DestroyWorkerThread(
         JetByteTools::Core::IThreadPoolWorkerThreadCallback &thread) override;

      void DisposeOfQueuedItemsAfterShutdown(
         ULONG_PTR completionKey,
         DWORD numberOfBytes,
         OVERLAPPED *pOverlapped) override;

      // Implement IThreadPoolWorkerThreadCallback

      bool Initialise() override;

      void Process(
         ULONG_PTR completionKey,
         DWORD numberOfBytes,
         OVERLAPPED *pOverlapped) override;

      void Shutdown() override;

      void OnProcessException(
         const JetByteTools::Core::_tstring & /*message*/) override {}

      /// Called if a thread is terminated due to an exception.

      void OnThreadTerminationException(
         const JetByteTools::Core::_tstring & /*message*/) override {}

      // Implement IMonitorThreadPool

      void OnThreadPoolThreadStarted() override;

      void OnThreadPoolDispatch() override;

      void OnThreadPoolProcessingStarted(
         bool dataAvailable) override;

      void OnThreadPoolProcessingStopped(
         bool dataAvailable) override;

      void OnThreadPoolThreadStopped() override;

      void OnThreadPoolError(
         const JetByteTools::Core::_tstring & /*message*/) override {}

      JetByteTools::Core::CThreadPool m_threadPool;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;
};

#endif // THREAD_POOL_SIMPLE_PROTOCOL_SERVER_THREAD_POOL_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ThreadPool.h
///////////////////////////////////////////////////////////////////////////////
